/** Header file for generator
 *
 * @author Ondrej Cekan
 * @date 20/02/2014
 * @update 05/07/2015
 *
     Copyright (C) 2015 Ondrej Cekan

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */


#include <string>
#include <vector>


using namespace std;


/**< Parser states */
enum pstate
{
    INIT,
  /**< Processor description */
    SUBS,
    SUBSSUBS,
    VARNAME,
    VAR,
    VARMIN,
    VARMAX,
    VARNUM,
    SYN,
    SYNSYNTAX,
  /**< Processor constraints */
    CONS,
    CONSSPECIAL,
    CONS1,
    CONS2,
    CONSFUNC1,
    CONSFUNC2
};


/**< Error states */
enum estate
{
    UKEYWORD,
    UWORD,
    UKNOWN,
    UVAR,
    USYN
};


/**< Type of constraint */
enum cons_type
{
    CNONE,
    CVAR,
    CSTR,
    CNUM,
    CSUB,
    CFUNC,
    CSYN
};


/**< Type of variable */
enum var_type
{
    VINT,
    VINT16,
    VINT14,
    VUINT,
    VSTR,
    VBOOL
};



/**< Data type structure for Constraint Item in constraint file */
typedef struct t_consit Tconsit;


/**< Data type structure for Variable in constraint file */
typedef struct t_var
{
    string name;
    var_type type;
    int vint;
    unsigned int vuint;
    string vstr;
    bool vbool;
    vector<Tconsit> cons;
    vector<Tconsit>::iterator consIt;
} Tvar;


/**< Data type structure for Syntax in configuration file */
typedef struct t_syn
{
    string name;
    string syntax;
    vector<Tconsit> cons;
    vector<Tconsit>::iterator consIt;
    vector<Tconsit>::reverse_iterator consItr;
    vector<pair<string, string> > val;      // buffer with upcoming values
    vector<pair<string, string> > eval;     // buffer with assigned values
} Tsyn;


/**< Structure for Constraint Item in constraint file */
struct t_consit
{
    Tvar *var;
    Tvar *var2;
    Tsyn *syn;
    int val;
    int val2;
    string str;
    string rfrom;
    string rto;
    cons_type type;
    string op;
};


/**< Data type structure for Substitute in configuration file */
typedef struct t_subs
{
    string name;
    unsigned is_input;
    vector<string> subs;
} Tsubs;
